#ifndef X11_With_ColorCC
#define X11_With_ColorCC
#include "X11_With_Color.hpp"

int Xwidth = 100;
int Xheight = 100;
int Xmargin = 10;
int Xscale = 2;

void Initialize_X(int width, int height, int margin, int scale){
        /* get the colors black and white (see section for details) */
        int i;
        unsigned long black,white;

        Xwidth = width;
        Xheight = height;
        Xmargin = margin;
        Xscale = scale;

        dis=XOpenDisplay((char *)0);
        screen=DefaultScreen(dis);
        visual_info.screen = screen;
        visual_list = XGetVisualInfo(dis,DefaultScreen(dis),&visual_info,&i);

        visual = XDefaultVisual(dis, screen);
        depth = DefaultDepth(dis, screen);

        black=BlackPixel(dis,screen);
        white=WhitePixel(dis,screen);
        background = black;
        foreground = foreground;

        hint.x = 10000; hint.y = 0;
        hint.width = width+2*margin; hint.height = height+2*margin;
        hint.flags = PPosition | PSize;

        Create_Colormap();

        setattributes.colormap = cmap;
        setattributes.background_pixel = BlackPixel(dis,screen);
        setattributes.border_pixel = WhitePixel(dis,screen);
        setattributes.event_mask = ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask;
        win = XCreateWindow(dis, DefaultRootWindow(dis),0,0, width+2*margin+30, height+2*margin, 5, depth, InputOutput, visual,
                        CWColormap | CWBackPixel | CWBorderPixel, &setattributes);

        XSetStandardProperties(dis,win,"CPM","Well",None,NULL,0,&hint);
        XGetWindowAttributes(dis,win,&attributes);
        XSelectInput(dis,win, ExposureMask|ButtonPressMask|KeyPressMask| StructureNotifyMask);
        if (DoesBackingStore(DefaultScreenOfDisplay(dis))) {
                setattributes.backing_store = WhenMapped;
                XChangeWindowAttributes(dis,win,CWBackingStore,&setattributes);
        }

        gc = XCreateGC(dis, win, 0, 0);

        XSetBackground(dis,gc,background);
        XSetForeground(dis,gc,foreground);
        XSelectInput(dis,win, ExposureMask|ButtonPressMask|KeyPressMask| StructureNotifyMask);

        // I support the WM_DELETE_WINDOW protocol
        Atom WM_DELETE_WINDOW = XInternAtom(dis, "WM_DELETE_WINDOW", False);
        XSetWMProtocols(dis, win, &WM_DELETE_WINDOW, 1);

        // XClearWindow(dis,win);
        XMapRaised(dis,win);
        XNextEvent(dis, &event);

        imagedata = (char *)calloc(width*height,depth);

        if (imagedata == NULL) printf("Error in memory allocation\n");
        planeimage = XCreateImage(dis,visual,depth,ZPixmap,0,imagedata,width,height,8,0);
        barimage = XCreateImage(dis,visual,depth,ZPixmap,0,imagedata,30,height,8,0);


};

void Close_X() {
        XFreeGC(dis, gc);
        XDestroyWindow(dis,win);
        XCloseDisplay(dis);
        XFreeColormap(dis,cmap);
};

void Redraw() {
        XClearWindow(dis, win);
};

void ColorRGB(int i,int r,int g,int b)
{
        UserCol[i][0] = 3;
        UserCol[i][1] = r;
        UserCol[i][2] = g;
        UserCol[i][3] = b;
};

void Create_Colormap() {
        int i, colormap_size;
        colormap_size = DisplayCells(dis, screen);

	ColorRGB(0,0,0,0);
        for(i=1;i<colormap_size/2;i++)
            ColorRGB(i,i*2,0,255);
        for(i=colormap_size/2;i<colormap_size-1;i++)
            ColorRGB(i,255,0,255-i*2);
    ColorRGB(253,0,255,100);
	ColorRGB(254,255,255,255);
	ColorRGB(255,60,60,60);

        if ((colors = (XColor *)calloc(colormap_size,sizeof(XColor))) == NULL) fprintf(stderr, "No memory for setting up colormap\n"), exit(1);

        for (i=0; i < colormap_size; i++)
        {
                colors[i].pixel = i;
                colors[i].flags = DoRed | DoGreen | DoBlue;
        }
        XQueryColors(dis,DefaultColormap(dis,screen),colors,colormap_size);
        for (i=0; i <= 255; i++)
        {
                if (i < colormap_size) {
                        colors[i].red = 257*UserCol[i][1];
                        colors[i].green = 257*UserCol[i][2];
                        colors[i].blue = 257*UserCol[i][3];
                }
                else fprintf(stderr,"Cannot set color %d, your screen has colormap size %d\n",i,colormap_size);
        }

        cmap = XDefaultColormap(dis,screen);

        for (i=0; i <= 255 && i < colormap_size; i++)
                XAllocColor(dis,cmap,&colors[i]);
};

int Write_Jpeg(XImage *img, const char* filename)
{
        FILE* outfile;
        unsigned long pixel;
        int x, y;
        char* buffer;
        struct jpeg_compress_struct cinfo;
        struct jpeg_error_mgr       jerr;
        JSAMPROW row_pointer;

        outfile = fopen(filename, "wb");
        if (!outfile) {
                fprintf(stderr, "Failed to open output file %s", filename);
                return 1;
        }

        /* collect separate RGB values to a buffer */
        buffer = (char*)malloc(sizeof(char)*3*img->width*img->height);
        for (y = 0; y < img->height; y++) {
                for (x = 0; x < img->width; x++) {
                        pixel = XGetPixel(img,x,y);
                        buffer[y*img->width*3+x*3+0] = (char)(pixel>>16);
                        buffer[y*img->width*3+x*3+1] = (char)((pixel&0x00ff00)>>8);
                        buffer[y*img->width*3+x*3+2] = (char)(pixel&0x0000ff);
                }
        }

        cinfo.err = jpeg_std_error(&jerr);
        jpeg_create_compress(&cinfo);
        jpeg_stdio_dest(&cinfo, outfile);

        cinfo.image_width = img->width;
        cinfo.image_height = img->height;
        cinfo.input_components = 3;
        cinfo.in_color_space = JCS_RGB;

        jpeg_set_defaults(&cinfo);
        jpeg_set_quality(&cinfo, 100, TRUE);
        jpeg_start_compress(&cinfo, TRUE);

        while (cinfo.next_scanline < cinfo.image_height)
	{
                row_pointer = (JSAMPROW) &buffer[cinfo.next_scanline
                        *(img->depth>>3)*img->width];
                jpeg_write_scanlines(&cinfo, &row_pointer, 1);
        }
        free(buffer);
        jpeg_finish_compress(&cinfo);
        fclose(outfile);

        return 0;
}

void InitializeGrid(int width, int height)
{
        int i = 0, j=0;
        int mar = 100;
        MyGrid = (int**)malloc(width*height*sizeof *MyGrid);
        for(i=0;i<width;i++)
        {
                MyGrid[i] = (int*)malloc(width * sizeof *MyGrid[i]);
                for(j=0;j<height;j++) MyGrid[i][j] = 200;
        }

}

void DrawBar(float avgy1, float avgy2)
{
	int i = 0, j=0;
	int linepos = (Xheight)-(Xheight)*avgy1*20/250;
    int linepos2 = (Xheight)-(Xheight)*avgy2*20/250;
    for(i=0;i<30;i++)
        for(j=0;j<Xheight;j++)
		{
			int color = (int)((float)j/Xheight*253);
            if(j==linepos || j == linepos+1)
			         XPutPixel(barimage,i,j,colors[254].pixel);
            else if(j==linepos2 || j == linepos2+1)
 			         XPutPixel(barimage,i,j,colors[0].pixel);
			else
				XPutPixel(barimage,i,j,colors[color].pixel);
		}
        XPutImage(dis,win,gc,barimage,0,0,Xmargin,Xmargin,(unsigned int)(Xwidth),(unsigned int)(Xheight));

}

void DisplayGrid(int time)
{
        int i = 0, j=0;
        for(i=0;i<Xwidth*Xscale;i++)
                for(j=0;j<Xheight*Xscale;j++)
                        XPutPixel(planeimage,i,j,colors[MyGrid[i/Xscale][j/Xscale]].pixel);
        XPutImage(dis,win,gc,planeimage,0,0,Xmargin+Xmargin+15,Xmargin,(unsigned int)(Xwidth),(unsigned int)(Xheight));
	sprintf(text, "images/%s%06d.jpg", "image", time);
	if(time%10==0)Write_Jpeg(planeimage, text);
}

void CatchEvents()
{
        while(XEventsQueued(dis,QueuedAfterFlush) > 0)
        {
                XNextEvent(dis, &event);
                if (event.type==Expose && event.xexpose.count==0) /* the window was exposed redraw it! */
		{
                        Redraw();
		}
                if (event.type==KeyPress&&XLookupString(&event.xkey,text,255,&key,0)==1)
                {  /* use the XLookupString routine to convert the invent KeyPress data into regular text.  Weird but necessary...*/
                        printf("You pressed the %c key!\n",text[0]);

                        switch(text[0])
                        {
                                case 'q':
					            printf("Quiting program...\n");
                                printf("(plots can be shown by using the command 'xmgrace -batch gracebat.bfile -noask')\n");
                                        Close_X();
                                        exit(0);
                                //case 't':
                                //       strcpy(text,"X can also do text!"); // You can also draw text! (but the grid now overwrites this)
                                //       XDrawString(dis,win,gc,(Xwidth/2)-30,15, text, strlen(text));
                                //       break;
                                case 's':
                                        printf("What name do you want for your jpg? ");
                                        scanf("%s", text);
                                        sprintf(text, "%s%s", text, ".jpg");
                                        printf("Saving to %s!\n",text);
                                        Write_Jpeg(planeimage, text);
                        }
                }
                if (event.type==ButtonPress) {            /* tell where the mouse Button was Pressed, could be useful for your program */
                        int x=event.xbutton.x, y=event.xbutton.y;
                        int gridx = -1; //(int)(x/scale);
                        int gridy = -1; //(int)(y/scale);
                        if(x>Xmargin && x <Xwidth+Xmargin)  gridx = (int)((x-Xmargin)/Xscale);
                        if(y>Xmargin && y <Xheight+Xmargin) gridy = (int)((y-Xmargin)/Xscale);
                        int j = 0, nc = 20;
                        int colortje = 255*255;
                        if(gridx>=0 && gridy>=0)
                        {
                                printf("You pressed the x, y coordinates: %d,%d!\n",gridx,gridy);
                                MyGrid[gridx][gridy] = 1;
                                XSetForeground(dis,gc,colortje);
                        }
                }
                if (event.type == ClientMessage)
                {
                        printf("You closed the window... so I'll stop the program\n");
                        printf("(plots can be shown by using the command 'xmgrace -batch gracebat.bfile -noask')\n");
                        Close_X();
                        exit(1);
                }
        }
}
#endif
